# Generated by roxygen2: do not edit by hand

S3method(print,session)
export(df2sf)
export(fix_geom)
export(geocode_osm)
export(geocode_osm_batch)
export(get_data)
export(get_info)
export(hot_spot)
export(line2poly)
export(make_ticker)
export(merge_list)
export(nesting)
export(point2poly_simp)
export(point2poly_tess)
export(poly2poly_ap)
export(sf2raster)
export(smart_round)
export(update_bbox)
export(utm_select)
import(Rcpp)
import(cartogram)
import(packcircles)
import(sf)
importFrom(RANN,nn2)
importFrom(cartogram,cartogram_cont)
importFrom(curl,curl_version)
importFrom(data.table,":=")
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,rbindlist)
importFrom(data.table,setnames)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,everything)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,last)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarize)
importFrom(httr,GET)
importFrom(httr,add_headers)
importFrom(httr,config)
importFrom(httr,content)
importFrom(httr,handle)
importFrom(httr,headers)
importFrom(httr,status_code)
importFrom(httr,warn_for_status)
importFrom(jsonlite,fromJSON)
importFrom(measurements,conv_unit)
importFrom(methods,as)
importFrom(purrr,reduce)
importFrom(rlang,abort)
importFrom(rmapshaper,ms_dissolve)
importFrom(sf,sf_use_s2)
importFrom(sf,st_area)
importFrom(sf,st_as_sf)
importFrom(sf,st_as_sfc)
importFrom(sf,st_bbox)
importFrom(sf,st_buffer)
importFrom(sf,st_cast)
importFrom(sf,st_centroid)
importFrom(sf,st_collection_extract)
importFrom(sf,st_combine)
importFrom(sf,st_convex_hull)
importFrom(sf,st_coordinates)
importFrom(sf,st_crs)
importFrom(sf,st_difference)
importFrom(sf,st_geometry)
importFrom(sf,st_intersection)
importFrom(sf,st_is)
importFrom(sf,st_is_valid)
importFrom(sf,st_make_valid)
importFrom(sf,st_transform)
importFrom(sf,st_union)
importFrom(spdep,localG)
importFrom(spdep,localmoran)
importFrom(spdep,nb2listw)
importFrom(spdep,poly2nb)
importFrom(stats,as.dist)
importFrom(stats,median)
importFrom(stats,weighted.mean)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_split)
importFrom(terra,as.data.frame)
importFrom(terra,crds)
importFrom(terra,ext)
importFrom(terra,extract)
importFrom(terra,mask)
importFrom(terra,ncell)
importFrom(terra,plot)
importFrom(terra,project)
importFrom(terra,rasterize)
importFrom(terra,res)
importFrom(terra,values)
importFrom(terra,vect)
importFrom(utils,URLencode)
importFrom(utils,data)
