#' \code{SUNGEO}
#'
#' Sub-National Geospatial Data Archive System: Geoprocessing Toolkit
#'
#' See the README on
#' [GitHub](https://github.com/zhukovyuri/SUNGEO#readme)
#'
#' @docType package
#' @name SUNGEO
#'
## quiets concerns of R CMD check re: the .'s that appear in pipelines
if(getRversion() >= "2.15.1"){
  utils::globalVariables(c("%>%",".",":=","AREA_W","POP_INT","POP_TOTAL","POP_W","TERM","alternatenames","asciiname","feature_code","geometry","geonameid","gn_type","latitude","longitude","match_type","name","sgeoz_length","strdist","area_code","col.id","overlap_1","overlap_2","action_dots","abort","vctrs_as_location","ST_MULTIPOINT","Field","Var","var_","Return_ID",".N",".SD","DATE","DATE_ALT","MID","POP_W_INV","TID","Varz","WID","X","Y","YEAR","YRMO","YRWK","osm_adm0_code","osm_adm0","osm_adm1","osm_adm2","osm_adm3","osm_adm4"))
  }
